@@ -15,6 +15,7 @@ from .graphql_helpers.schema_helpers import (
     graphql_query,
     graphql_update_or_create,
     merge_data_fields_on_update,
+    process_filter_kwargs,
     DENY,
     CREATE,
     UPDATE,
@@ -81,11 +82,6 @@ from .graphql_helpers.views import (
     SafeGraphQLView
 )
 
-from .schema import (
-    Query,
-    Mutation
-)
-
 from .testcases import (
     client_for_testing
 )
