@@ -19,7 +19,8 @@ from graphql.language import ast
 from graphql.language.printer import print_ast
 from inflection import camelize
 from rescape_python_helpers import ramda as R, memoize
-from rescape_python_helpers.functional.ramda import to_dict_deep, flatten_dct, to_pairs, flatten_dct_until
+from rescape_python_helpers.functional.ramda import to_dict_deep, flatten_dct, to_pairs, flatten_dct_until, \
+    to_array_if_not
 
 from .graphene_helpers import dump_graphql_keys, dump_graphql_data_object, camelize_graphql_data_object, call_if_lambda
 
@@ -55,7 +56,9 @@ DELETE = 'delete'
 # From django-filters. Whenever graphene supports filtering without Relay we can get rid of this here
 # Educated guesss about what types for each to support. Django/Postgres might support fewer or more of these
 # combinations than I'm aware of
-# Skip if testing. These take forever
+# Skip most if testing. These take forever
+# Results in a dict keyed by the filter suffix and valued by a dict of the allowed types and possibly a type_modifier
+# lambda to create a grapheene.List of the given type for the 'in' and 'range' suffixes
 FILTER_FIELDS = R.compose(
     # Create not versions of each. This is a pseudo syntax not supported by Django. Django uses exclude() or ~Q(expr).
     # We can't create an equivalent here with keys, so we use _not and convert it to ~Q(expr) later
@@ -157,16 +160,18 @@ class DataPointRelatedCreateInputType(InputObjectType):
 
 
 def _memoize(args):
-   return [
-    # Only use graphene_type here. type is a function and can't be serialized
-    args[0]['graphene_type'],
-    args[1],
-    # TODO We use the parent_type_class to make each type unique. I don't know why graphene won't let us reuse
-    # input types within the schema. It seems like a UserInputType should be reusable whether it's the User
-    # of a Region or the user of a Group.
-    args[2] if R.lte(3, R.length(args)) and R.isinstance((list, tuple), args[2]) else [args[2]],
-    args[3] if R.lte(2, R.length(args)) else False
-]
+    return [
+        # Only use graphene_type here. type is a function and can't be serialized
+        args[0]['graphene_type'],
+        args[1],
+        # TODO We use the parent_type_class to make each type unique. I don't know why graphene won't let us reuse
+        # input types within the schema. It seems like a UserInputType should be reusable whether it's the User
+        # of a Region or the user of a Group.
+        args[2] if R.lte(3, R.length(args)) and R.isinstance((list, tuple), args[2]) else [args[2]],
+        args[3] if R.lte(2, R.length(args)) else False
+    ]
+
+
 @memoize(map_args=_memoize)
 def input_type_class(field_config, crud, parent_type_classes=[], allowed_fields_only=False):
     """
@@ -523,6 +528,7 @@ def top_level_allowed_filter_arguments(fields, graphene_type):
     """
     return input_type_class(dict(fields=fields, graphene_type=graphene_type), 'read', [], True)
 
+
 def allowed_filter_arguments(fields_dict, graphene_type):
     """
         Like allowed_query_and_read_arguments but only used for arguments and adds filter variables like id_contains.
@@ -630,7 +636,6 @@ def instantiate_graphene_type(field_config, parent_type_classes, crud):
         )
 
 
-
 def input_type_fields(fields_dict, crud, parent_type_classes=[]):
     """
     :param fields_dict: The fields_dict for the Django model or json data
@@ -787,11 +792,12 @@ def graphql_query(graphene_type, fields, query_name):
                 )
             )
         )
-        query = print_ast(parse('''query someMadeUpString%s { 
+        query = print_ast(parse('''query %s%s { 
                 %s%s {
                     %s
                 }
             }''' % (
+            query_name,
             '(%s)' % formatted_definitions if formatted_definitions else '',
             query_name,
             '(%s)' %
@@ -814,7 +820,7 @@ def graphql_query(graphene_type, fields, query_name):
             ),
             kwargs
         )
-        logger.debug(f'Query: {query}\nKwargs: {R.dump_json(camelized_kwargs)}')
+        logger.debug(f'Query: {query}\nKwargs: {R.dump_json(R.prop("variables", camelized_kwargs))}')
         return client.execute(
             query,
             **camelized_kwargs
@@ -925,8 +931,20 @@ def process_query_value(model, value_dict):
         )
     )
 
-def not_ends_with_contains(key):
-    return not key.endswith('contains')
+
+def _flatten_until(key, value):
+    return R.isinstance(str, key) and not R.isinstance(list, value)
+
+
+def _key_matches_filter_field(key):
+    """
+        Returns true if the key string ends with a FILTER_FIELD like contains or range or in
+    :param key:
+    :return:
+    """
+    last_key = R.last(key.split('__'))
+    return R.has(last_key, FILTER_FIELDS)
+
 
 def process_query_kwarg(model, key, value):
     """
@@ -943,7 +961,7 @@ def process_query_kwarg(model, key, value):
     :return: A list of Q expressions like Q(x__y=1) and ~Q(x__contains='adf')
     """
 
-    if key.endswith('__contains'):
+    if False and key.endswith('__contains'):
         # Don't modify json contains searches
         # TODO this doesn't make sense because condition below is supposed to handle it
         return [Q(**{key: value})]
@@ -952,27 +970,46 @@ def process_query_kwarg(model, key, value):
         k = key.replace('__not', '')
         return [~Q(**{k: value})]
     if isinstance(R.prop_or(None, key, model._meta._forward_fields_map), (JSONField,)):
-        # Small correction here to change the data filter to data__contains to handle any json
-        # https://docs.djangoproject.com/en/2.0/ref/contrib/postgres/fields/#std:fieldlookup-hstorefield.contains
-        # This is just one way of filtering json. We can also do it with the argument structure
         return R.compose(
-            lambda dct: R.map_with_obj_to_values(lambda key, value: Q(**{key: value}), dct),
-            lambda dct: flatten_dct_until(dct, lambda key: not isinstance(str, key) and not_ends_with_contains(key), '__')
+            lambda dct: R.map_with_obj_to_values(
+                lambda key, value: Q(**{key: value}),
+                dct
+            ),
+            # Small correction here to change the data filter to data... to data...__contains to handle any json
+            # https://docs.djangoproject.com/en/2.0/ref/contrib/postgres/fields/#std:fieldlookup-hstorefield.contains
+            # If the value is an object or array,
+            # add __contains to the end there isn't already a filter suffix
+            # __contains allows matching an object or array without the array orders having to match
+            lambda dct: R.map_keys_with_obj(
+                lambda key, value: R.when(
+                    lambda key: isinstance(value, (dict, list)) and not _key_matches_filter_field(key),
+                    lambda key: R.join('__', [key, 'contains'])
+                )(key),
+                dct
+            ),
+            lambda dct: flatten_dct_until(
+                dct,
+                _flatten_until,
+                '__'
+            )
         )({key: value})
     elif R.has(key, model._meta._forward_fields_map):
         # If it's a model key
         if isinstance(model._meta._forward_fields_map[key], (ForeignKey, OneToOneField, ManyToManyField)):
             # Recurse on these, so foo: {bar: 1, car: 2} resolves to [['foo__bar' 1], ['foo__car', 2]]
             related_model = model._meta._forward_fields_map[key].related_model
-            # values have to be arrays, so iterate over them
-            q_expressions = R.chain(lambda v: process_query_value(related_model, v), value)
+            # Convert the values to array if not and chain to flatten
+            q_expressions = R.chain(
+                lambda v: process_query_value(related_model, v),
+                to_array_if_not(value)
+            )
             return R.map(
                 # TODO we assume simple Q expressions with ony one child at children[0]
                 lambda q_expression: Q(**{f'{key}__{q_expression.children[0][0]}': q_expression.children[0][1]}),
                 q_expressions
             )
         elif isinstance(model._meta._forward_fields_map[key], (ManyToManyField)):
-            raise NotImplementedError("TODO need to implement stringify for ManyToMany")
+            raise NotImplementedError(f'Unrecognized field type for {model._meta._forward_fields_map[key]}')
     elif R.contains(R.head(key.split('__')), R.map(R.prop('name'), model._meta.related_objects)):
         # Reverse Many-to-Many relationships, the only thing I know how to support at this point is
         # __in=[ids], although it might be possible to support other filters
@@ -1103,3 +1140,87 @@ def deep_merge_existing_json(django_model, json_prop, data):
         )
     # Otherwise just return the new value if any
     return R.prop_or(None, json_prop, data)
+
+
+def invert_q_expressions_sets(q_expressions):
+    """
+        Index q_expressions by their key (children[0][0]) into sets, then
+        inverts the matrix of q_expressions into 1 or 2 sets. The first set is most and the 2nd set
+        ist the second many-to-many expression many_to_many_intersection_streets_to_and_statement
+        that must be run as a second filter
+    :param q_expressions: list q_expressions. Only supports simple Q expressions with one clause
+    :return: 1 or 2 sets of q_expressions that can be used by sequential filters
+    """
+
+    return R.compose(
+        # Sort by set index and return the values
+        lambda index_dicts: R.map(
+            lambda key: R.compose(
+                lambda values: R.map(R.prop('q_expression'), values),
+                lambda key: R.prop(key, index_dicts)
+            )(key),
+            sorted(R.keys(index_dicts))
+        ),
+        # Group by index within each set. Most will be 0. Only then second many-to-many can be 1
+        lambda dcts: R.index_by(R.prop('index'), dcts),
+        lambda q_expression_sets: R.chain(
+            lambda q_expression_set: R.map(
+                lambda i_q_expression: dict(index=i_q_expression[0], q_expression=i_q_expression[1]),
+                enumerate(q_expression_set)
+            ),
+            q_expression_sets
+        ),
+        # One special case we need to handle multiple 'intersections_data_streets'
+        # We need to sequentially filter
+        # We do this by grouping them together into an array here, then we invert the 2D array
+        lambda q_expressions: R.values(
+            R.index_by(lambda q_expression: q_expression.children[0][0], q_expressions)
+        )
+    )(q_expressions)
+
+
+def process_filter_kwargs_with_to_manys(model, kwargs):
+    """
+        Calls process_filter_kwargs and then invert_q_expressions_sets so that
+        the filtering can correctly handle to-many relations. This works
+        by grouping the same keys together so that a search for two values of a to-many
+        property must both be present (most often one is present in each of two to-mnay instances)
+    :param model: The django model
+    :param kwargs: The kwargs to filter by
+    :return: Sets of q_expressions that are run sequentially
+    """
+    return R.compose(
+        lambda q_expressions: invert_q_expressions_sets(q_expressions),
+        lambda kwargs: process_filter_kwargs(model, kwargs)
+    )(kwargs)
+
+
+def query_sequentially(manager, manager_method, q_expressions_sets):
+    """
+        Sequentially queries the q_expression_sets formed by  process_filter_kwargs_with_to_manys/invert_q_expressions_sets
+        Sequentially querying allows toMany values to be sought when multiple matching toMany instances are sought
+    :param manager: The django model manager
+    :param manager_method: The django model manager method. Normally 'filter', but could be 'count' or 'get'
+    For sequential queries only the last query can be anything other than filter
+    :param q_expressions_sets: List of lists of q expressions, where common query paths are separated into
+    different sets so they can be run sequentially (see invert_q_expressions_sets)
+    :return: The query response
+    """
+
+    if not R.length(q_expressions_sets):
+        return getattr(manager, manager_method)()
+
+    def _reduce(last, mgr_or_queryset, q_expressions_set):
+        # We need distinct because the intersections__data__streets can generate duplicates of the same location
+        return getattr(mgr_or_queryset, manager_method if last else 'filter')(*q_expressions_set).distinct()
+
+    last = R.last(q_expressions_sets)
+    return R.reduce(
+        lambda manager_or_query, q_expressions: _reduce(
+            R.equals(q_expressions, last),
+            manager_or_query,
+            q_expressions
+        ),
+        manager,
+        q_expressions_sets
+    )
